package com.fastbee.ggroup.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.fastbee.common.utils.DateUtils;
import com.fastbee.ggroup.domain.GLegendType;
import com.fastbee.ggroup.domain.dto.GLegendUpdateDto;
import com.fastbee.ggroup.domain.dto.GLegendCateGoryDto;
import com.fastbee.ggroup.domain.dto.GLegendDto;
import com.fastbee.ggroup.mapper.GLegendTypeMapper;
import com.github.yulichang.wrapper.MPJLambdaWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fastbee.ggroup.mapper.GLegendMapper;
import com.fastbee.ggroup.domain.GLegend;
import com.fastbee.ggroup.service.IGLegendService;
import org.springframework.transaction.annotation.Transactional;

/**
 * 图例Service业务层处理
 *
 * @author kerwincui
 * &#064;date  2024-10-08
 */
@Service
@Slf4j
public class GLegendServiceImpl implements IGLegendService
{
    @Autowired
    private GLegendMapper gLegendMapper;

    @Autowired
    private GLegendTypeMapper gLegendTypeMapper;

    /**
     * 查询图例
     *
     * @param id 图例主键
     * @return 图例
     */
    @Override
    public GLegendDto selectGLegendById(Long id)
    {
        return gLegendMapper.selectJoinOne(GLegendDto.class,new MPJLambdaWrapper<GLegend>()
                .select(GLegend::getId,GLegend::getIcon,GLegend::getType,GLegend::getName)
                .select(GLegendType::getTypeName,GLegendType::getCategory)
                .leftJoin(GLegendType.class,GLegendType::getId,GLegend::getType)
                .eq(GLegend::getId,id));
    }

    /**
     * 查询/搜索图例列表
     * @param gLegend 图例
     * @return 图例
     */
    @Override
    public List<GLegendDto> selectGLegendList(GLegend gLegend) {
        return gLegendMapper.selectJoinList(GLegendDto.class, new MPJLambdaWrapper<GLegend>()
                .select(GLegend::getId, GLegend::getIcon, GLegend::getType, GLegend::getName)
                .select(GLegendType::getTypeName, GLegendType::getCategory)
                .leftJoin(GLegendType.class, GLegendType::getId, GLegend::getType)
                .like(StringUtils.isNotBlank(gLegend.getSearchValue()),
                        GLegend::getName, StringUtils.isNotBlank(gLegend.getSearchValue()) ?
                                gLegend.getSearchValue() : null));
    }

    /**
     * 新增图例
     *
     * @param gLegend 图例
     * @return 结果
     */
    @Override
    public int insertGLegend(GLegend gLegend)
    {
        return gLegendMapper.insertGLegend(gLegend);
    }

    /**
     * 修改图例
     *
     * @param gLegend 图例
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateGLegend(GLegendUpdateDto gLegend)
    {
        return  (gLegendTypeMapper.updateGLegendType(GLegendType.builder()
                .id( gLegend.getType())
                .category(gLegend.getCategory())
                .updateTime(DateUtils.getNowDate())
                .build())==1&&
        gLegendMapper.updateGLegend( GLegend.builder()
                        .id(gLegend.getId())
                        .icon(gLegend.getIcon())
                        .name(gLegend.getName())
                        .type(gLegend.getType())
                        .updateTime(DateUtils.getNowDate())
                .build())==1)?1:0;
    }
    /**
     * 批量删除图例
     *
     * @param ids 需要删除的图例主键
     * @return 结果
     */
    @Override
    public int deleteGLegendByIds(Long[] ids)
    {
        return gLegendMapper.deleteGLegendByIds(ids);
    }

    /**
     * 删除图例信息
     *
     * @param id 图例主键
     * @return 结果
     */
    @Override
    public int deleteGLegendById(Long id)
    {
        return gLegendMapper.deleteGLegendById(id);
    }

    /**
     * 按照类别返回所有类别下图例列表
     * @return 图例列表
     */
    @Override
    public List<Map<String, Object>> getLegendListByCategory() {
        // 按照类别分组，并生成结果列表
        return gLegendMapper.selectJoinList(GLegendCateGoryDto.class, new MPJLambdaWrapper<GLegend>()
                .select(GLegend::getIcon, GLegend::getName)
                .select(GLegendType::getTypeName, GLegendType::getCategory)
                .leftJoin(GLegendType.class, GLegendType::getId, GLegend::getType)).stream()
                .collect(Collectors.groupingBy(GLegendCateGoryDto::getCategory))
                .entrySet().stream()
                .map(entry -> {
                    Map<String, Object> itemMap = new HashMap<>();
                    itemMap.put("category", entry.getKey());
                    itemMap.put("list", entry.getValue());
                    return itemMap;
                })
                .collect(Collectors.toList());
    }


}
